#!/bin/bash

# This script is designed to take advantage of randomise to perform non-parametric permutation testing to non-imaging data. It should be especially convenient to FSL users, and we recommend the use of FSL Glm to generate the design/contrast/f-test/permutation block files
# 
# Three basic input files are required:
#	1. Interest variable matrix: text file containing the variables to be tested, consisting of one column per variable and one row for each observation
#	2. Design matrix (.mat) 
#	3. Contrast matrix (.con)
#
# By default, p values are uncorrected for multiple comparisons. Optionally, family-wise error corrected p values are generated by comparing the statistic of each variable to the maximum statistic distribution across all variables under the null hypothesis - makes sense as long as the variables included can be considered as a family
# 
# Additional options include:
#	1. F tests: requires .fts files (with the same root name as the design and contrast files)
#	2. Block permutation: requires exchangeability block labels .grp file (with the same root name as the design and contrast files)
#
# Output files:
#	1. P value file: named <output>_p_all_contrasts
#	2. Stats file: named <output>_stat_all_contrasts
#	3. F test p value file: <output>_p_F_test
#	4. F test stats file: <output>_Fstat
#	5. Corrected p value file: <output>_corrp_all_contrasts
#	6. Corrected F test p value file: <output>_corrp_F_test


# define compulsory and optional arguments
while getopts ":V:D:O:P:G:C:F:" o; do
    case "${o}" in
        V)
            V=${OPTARG}
            ;;
        D)
            D=${OPTARG}
            ;;
        O)
            O=${OPTARG}
            ;;
        P)
            P=${OPTARG}
            ;;
        G)
            G=${OPTARG}
            ;;
        C)
            C=${OPTARG}
            ;;
        F)
            F=${OPTARG}
            ;;
    esac
done

if [ -z "${V}" ] || [ -z "${D}" ] || [ -z "${O}" ] || [ -z "${O}" ]; then printf "\n	ERROR: missing non-optional argument(s) \n\n Compulsory arguments: -V <parameter name (variables matrix)> -D <design (and cotrast) matrix name> -O <output base name>\n\n Optional arguments:\n	-P: number of permutations (default 10,000)\n	-G: perform block permutation - YES or NO (default NO; grp block labels file must have been created) \n	-C: get FWE-corrected p values (to be used with care) - YES or NO (default NO)\n	-F: perform F test - YES or NO (default NO; fts file must have been created)\n\n 	****WARNING: no more than 9 contrasts should be included in the contrast matrix\n\n	(c) 2017 Grup de Neuropsicologia\n	Universitat de Barcelona\n	by Alexandra Abos and Hugo C Baggio\n\n"; exit 1; fi

rand_id=`head -200 /dev/urandom | cksum | cut -f1 -d " "`

mkdir tempNIR_${rand_id}
cp ${route_NIR}/parameter2nifti.py tempNIR_${rand_id}/

parameter_name=$V
design=$D
randomise_output=$O
nifti=nifti_${randomise_output}

if [ ! -z "${P}" ]; then perm=$P; else perm=10000; fi

if [ ! -z "${C}" ]; then get_FWE_corrected_p=$C; else get_FWE_corrected_p=NO; fi

if [ ! -z "${F}" ]; then f_test=$F; else f_test=NO; fi

if [ ! -z "${G}" ]; then blocks=$G; else blocks=NO; fi

cp ${parameter_name} tempNIR_${rand_id}/temp_val

cd tempNIR_${rand_id}

	# Get contrast names and number
	grep "/NumContrasts" ../${design}.con > tempNIR_number_contrasts1_${randomise_output}
	cut -c15- tempNIR_number_contrasts1_${randomise_output} > tempNIR_number_contrasts2_${randomise_output}

	number_contrasts=`cat tempNIR_number_contrasts2_${randomise_output}`

	if [ "${number_contrasts}" -gt 9 ]; then printf "\n>>>>>>	ERROR - more than 9 contrasts specified!!!!!\n\n"; exit; fi

	head -${number_contrasts} ../${design}.con > tempNIR_contrast_columnsA_${randomise_output}
	cut -c15- tempNIR_contrast_columnsA_${randomise_output} > tempNIR_contrast_columns1_${randomise_output}


	awk '{ for (f = 1; f <= NF; f++) a[NR, f] = $f }  NF > nf { nf = NF } END { for (f = 1; f <= nf; f++) for (r = 1; r <= NR; r++) printf a[r, f] (r==NR ? RS : FS) }' tempNIR_contrast_columns1_${randomise_output} > tempNIR_contrast_columns_${randomise_output}


	echo 'Parameter to nifti'
	python -c 'import parameter2nifti; parameter2nifti.parameter2nifti("temp_val")'

	echo 'Randomise'

	mv temp_val ./tempNIR_${nifti}
	mv temp_val.nii.gz ./tempNIR_${nifti}.nii.gz

	if [[ "${blocks}" == YES ]]
		then 
		if [[ "${f_test}" == YES ]]

			then
			echo "Performing F test"
			randomise -i tempNIR_${nifti}.nii.gz -d ../${design}.mat -t ../${design}.con -o tempNIR_${randomise_output} --uncorrp -x -n ${perm} -e ../${design}.grp -f ../${design}.fts
			else
			randomise -i tempNIR_${nifti}.nii.gz -d ../${design}.mat -t ../${design}.con -o tempNIR_${randomise_output} --uncorrp -x -n ${perm} -e ../${design}.grp
		fi
		else 
		if [[ "${f_test}" == YES ]]	
			then
			echo "Performing F test"
			randomise -i tempNIR_${nifti}.nii.gz -d ../${design}.mat -t ../${design}.con -o tempNIR_${randomise_output} --uncorrp -x -n ${perm} -f ../${design}.fts
			else
			randomise -i tempNIR_${nifti}.nii.gz -d ../${design}.mat -t ../${design}.con -o tempNIR_${randomise_output} --uncorrp -x -n ${perm}
		fi
	fi


	# Get p values - image inverted with fslmaths so as not to output 1-p

	for c in tempNIR_${randomise_output}_vox_p_tstat*.gz; do fslmaths ${c} -sub 1 -mul -1 tempNIR_inv_${c}; rm -f ${c%.nii.gz}_p_value; fslslice tempNIR_inv_${c} tempNIR_inv_${c}; for d in tempNIR_inv_${c%.nii.gz}_slice*; do fslstats ${d} -M >> ${c%.nii.gz}_p_value; done; done

	for c in tempNIR_${randomise_output}_vox_p_tstat*_p_value; do echo $c >> tempNIR_p_list_${randomise_output}; done

	rm -f ${randomise_output}_p_all_contrasts

	paste `cat tempNIR_p_list_${randomise_output}` >> tempNIR_${randomise_output}_p_all_contrasts1
	rm *${randomise_output}*slice*


	if [[ "${f_test}" == YES ]]
		then
		for c in tempNIR_${randomise_output}_vox_p_fstat*.gz; do fslmaths ${c} -sub 1 -mul -1 tempNIR_inv_${c}; rm -f ${c%.nii.gz}_p_value_f; fslslice tempNIR_inv_${c} tempNIR_inv_${c}; for d in tempNIR_inv_${c%.nii.gz}_slice*; do fslstats ${d} -M >> ${c%.nii.gz}_p_value_f; done; done

		for c in tempNIR_${randomise_output}_vox_p_fstat*_p_value_f; do echo $c >> tempNIR_p_f_list_${randomise_output}; done

		rm -f tempNIR_${randomise_output}_p_f_all_contrasts

		paste `cat tempNIR_p_f_list_${randomise_output}` >> tempNIR_${randomise_output}_p_f_all_contrasts1
		rm *${randomise_output}*slice*
	fi

	# Get corrected p values

	if [[ "${get_FWE_corrected_p}" == YES ]] 
		then
	
		for c in tempNIR_${randomise_output}_vox_corrp_tstat*.gz; do fslmaths ${c} -sub 1 -mul -1 tempNIR_inv_${c}; rm -f ${c%.nii.gz}_corrp_value; fslslice tempNIR_inv_${c} tempNIR_inv_${c}; for d in tempNIR_inv_${c%.nii.gz}_slice*; do fslstats ${d} -M >> ${c%.nii.gz}_corrp_value; done; done

		for c in tempNIR_${randomise_output}_vox_corrp_tstat*_corrp_value; do echo $c >> tempNIR_corrp_list_${randomise_output}; done

		rm -f tempNIR_${randomise_output}_corrp_all_contrasts

		paste `cat tempNIR_corrp_list_${randomise_output}` >> tempNIR_${randomise_output}_corrp_all_contrasts1


		# Corrected p - F test
		if [[ "${f_test}" == YES ]]
			then
			for c in tempNIR_${randomise_output}_vox_corrp_fstat*.gz; do fslmaths ${c} -sub 1 -mul -1 tempNIR_inv_${c}; rm -f ${c%.nii.gz}_p_value_f; fslslice tempNIR_inv_${c} tempNIR_inv_${c}; for d in tempNIR_inv_${c%.nii.gz}_slice*; do fslstats ${d} -M >> ${c%.nii.gz}_p_value_f; done; done

			for c in tempNIR_${randomise_output}_vox_corrp_fstat*_p_value_f; do echo $c >> tempNIR_corrp_f_list_${randomise_output}; done

			rm -f tempNIR_${randomise_output}_corrp_f_all_contrasts

			paste `cat tempNIR_corrp_f_list_${randomise_output}` >> tempNIR_${randomise_output}_corrp_f_all_contrasts1
			rm *${randomise_output}*slice*
		fi

	fi

	# Get stats values
	echo 'Getting stats'
	for c in tempNIR_${randomise_output}_tstat*.gz; do rm -f ${c%.nii.gz}_t_value; fslslice ${c} ${c}; for d in ${c%.nii.gz}_slice*; do fslstats ${d} -M >> ${c%.nii.gz}_t_value; done; done

	for c in tempNIR_${randomise_output}_tstat*_t_value; do echo $c >> tempNIR_t_list_${randomise_output}; done

	paste `cat tempNIR_t_list_${randomise_output}` >> tempNIR_${randomise_output}_stat_all_contrasts1

	if [[ "${f_test}" == YES ]]
		then
		for c in tempNIR_${randomise_output}_fstat*.gz; do rm -f ${c%.nii.gz}_F_value; fslslice ${c} ${c}; for d in ${c%.nii.gz}_slice*; do fslstats ${d} -M >> ${c%.nii.gz}_F_value; done; done

		for c in tempNIR_${randomise_output}_fstat*_F_value; do echo $c >> tempNIR_F_list_${randomise_output}; done

		#rm -f ${randomise_output}_Fstat_all_contrasts

		paste `cat tempNIR_F_list_${randomise_output}` >> tempNIR_${randomise_output}_Fstat_all_contrasts1
		rm *${randomise_output}*slice*
	fi
	rm -f ${randomise_output}_fstat*_F_value

	# Add first row
	for c in tempNIR_contrast_columns_${randomise_output} tempNIR_${randomise_output}_p_all_contrasts1
		do cat $c >> tempNIR_${randomise_output}_p_all_contrasts2
	done

	echo 'Getting p values'
	if [[ "${get_FWE_corrected_p}" == YES ]] 
		then
		for c in tempNIR_contrast_columns_${randomise_output} tempNIR_${randomise_output}_corrp_all_contrasts1
			do cat $c >> tempNIR_${randomise_output}_corrp_all_contrasts2
		done
	fi

	for c in tempNIR_contrast_columns_${randomise_output} tempNIR_${randomise_output}_stat_all_contrasts1
		do cat $c >> tempNIR_${randomise_output}_stat_all_contrasts2
	done

	# Remove tabs

	tr -d '\t' < tempNIR_${randomise_output}_p_all_contrasts2 > ../${randomise_output}_p_all_contrasts
	tr -d '\t' < tempNIR_${randomise_output}_stat_all_contrasts2 > ../${randomise_output}_stat_all_contrasts

	if [[ "${f_test}" == YES ]]
		then
		tr -d '\t' < tempNIR_${randomise_output}_p_f_all_contrasts1 > ../${randomise_output}_p_F_test
		tr -d '\t' < tempNIR_${randomise_output}_Fstat_all_contrasts1 > ../${randomise_output}_Fstat

		if [[ "${get_FWE_corrected_p}" == YES ]] 
			then
			tr -d '\t' < tempNIR_${randomise_output}_corrp_f_all_contrasts1 > ../${randomise_output}_corrp_F_test
		fi

	fi

	if [[ "${get_FWE_corrected_p}" == YES ]] 
		then
		tr -d '\t' < tempNIR_${randomise_output}_corrp_all_contrasts2 > ../${randomise_output}_corrp_all_contrasts
	fi

cd ..

rm -r tempNIR_${rand_id}



